package com.qunzhixinxi.hnqz.admin.config;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysNotice;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUserRole;
import com.qunzhixinxi.hnqz.admin.api.entity.WmPackageRedemptionNotice;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysNoticeService;
import com.qunzhixinxi.hnqz.admin.service.SysUserRoleService;
import com.qunzhixinxi.hnqz.admin.service.WmPackageRedemptionNoticeService;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 系统定时任务
 *
 * @author jimmy
 * @date 2024-11-21 17:11
 */
@Slf4j
@Configuration
@EnableScheduling
@AllArgsConstructor
public class ScheduleConfiguration {

    private final SysDeptService deptService;
    private final SysUserRoleService userRoleService;
    private final SysNoticeService noticeService;
    private final WmPackageRedemptionNoticeService packageRedemptionNoticeService;


    /**
     * 每天凌晨1点中 企业是否过期
     */
    @Async
    @Scheduled(cron = "0 0 1 * * *")
    public void checkEntExpiryDaily() {
        log.info("=================[checkEntExpiryDaily start]=================");

        try {
            List<SysDept> depts = deptService.list(Wrappers.<SysDept>lambdaQuery().eq(SysDept::getLevel, 4).eq(SysDept::getDelFlag, CommonConstants.STATUS_NORMAL));

            if (CollUtil.isEmpty(depts)) {
                return;
            }

            // 过滤当前日期到截止日期少于30天的
            LocalDate today = LocalDate.now();
            List<SysDept> collect = depts.stream().filter(d ->
                    {
                        LocalDate expiryDate = d.getExpiryDate();
                        return expiryDate != null && (today.plusMonths(1).isAfter(expiryDate));
                        // return true;
                    }
            ).collect(Collectors.toList());

            if (CollUtil.isEmpty(collect)) {
                return;
            }

            // 封装为消息，推送给区域管理员 roleId = 4

            List<SysUserRole> userRoles = userRoleService.list(Wrappers.<SysUserRole>lambdaQuery().eq(SysUserRole::getRoleId, 4));

            if (CollUtil.isEmpty(userRoles)) {
                return;
            }

            List<Integer> userIds = userRoles.stream().mapToInt(SysUserRole::getUserId).boxed().distinct().sorted().collect(Collectors.toList());


            LocalDateTime now = LocalDateTime.now();
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DatePattern.CHINESE_DATE_PATTERN);
            List<SysNotice> notices = new LinkedList<>();
            for (Integer userId : userIds) {

                for (SysDept dept : collect) {
                    SysNotice notice = new SysNotice();
                    notice.setTitle(String.format("【%s】即将到期", dept.getName()));
                    notice.setContent(String.format("【%s】服务时间将于【%s】到期，请关注！", dept.getName(), formatter.format(dept.getExpiryDate())));
                    notice.setUserId(userId);
                    notice.setReadFlag(false);
                    notice.setCreateBy("admin");
                    notice.setCreateTime(now);
                    notices.add(notice);
                }

            }

            if (CollUtil.isEmpty(notices)) {
                log.info("<<<<<< 不存在即将到期的服务商 >>>>>>");
            } else {
                noticeService.saveBatch(notices);
                log.info("<<<<<< 存在即将到期的服务商，计划发送通知{}条 >>>>>>", notices.size());
            }
        } finally {
            log.info("=================[checkEntExpiryDaily end  ]=================");
        }


    }


    /**
     * 每天凌晨2点，自动确认执行包兑付通知
     */
    @Async
    @Scheduled(cron = "0 0 2 * * *")
    public void autoConfirmPackageRedemptionNotice() {

        log.info("=================[autoConfirmPackageRedemptionNotice start ]=================");

        try {
            LocalDate lastDay = LocalDate.now().minusDays(1);
            LocalDateTime startTime = LocalDateTime.of(lastDay.minusDays(30), LocalTime.MIN);
            LocalDateTime endTime = LocalDateTime.of(lastDay, LocalTime.MAX);

            // 查询所有审核通过的兑付通知
            List<WmPackageRedemptionNotice> notices = packageRedemptionNoticeService.list(Wrappers.<WmPackageRedemptionNotice>lambdaQuery()
                    .eq(WmPackageRedemptionNotice::getCheckState, WmPackageRedemptionNotice.CheckState.PASSED.getState())
                    .between(WmPackageRedemptionNotice::getCreateTime, startTime, endTime)
                    .isNull(WmPackageRedemptionNotice::getAccept)
                    .orderByDesc(WmPackageRedemptionNotice::getCreateTime));

            if (CollUtil.isEmpty(notices)){
                return;
            }

            String autoConfirmMsg = "兑付通知超过30天未确认，系统默认确认通过";
            List<WmPackageRedemptionNotice> collect = notices.stream().map(n -> {
                WmPackageRedemptionNotice notice = new WmPackageRedemptionNotice();
                notice.setNoticeId(n.getNoticeId());
                notice.setNoticeState(WmPackageRedemptionNotice.NoticeState.CONFIRM.getState());
                notice.setAccept(true);
                notice.setAcceptTime(LocalDateTime.now());
                notice.setAcceptRemarks(autoConfirmMsg);
                notice.setUpdateBy("admin");
                notice.setUpdateTime(LocalDateTime.now());
                return notice;

            }).collect(Collectors.toList());


            packageRedemptionNoticeService.updateBatchById(collect);


        } finally {
            log.info("=================[autoConfirmPackageRedemptionNotice end  ]=================");
        }


    }
}
