package com.qunzhixinxi.hnqz.admin.manager;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.text.StrPool;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.dto.SysUserDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.SysFile;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUserCertificate;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUserDrug;
import com.qunzhixinxi.hnqz.admin.api.entity.WmDaDrugEntDrugtable;
import com.qunzhixinxi.hnqz.admin.service.SysFileService;
import com.qunzhixinxi.hnqz.admin.service.SysUserCertificateService;
import com.qunzhixinxi.hnqz.admin.service.SysUserDrugService;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntDrugtableService;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 用户管理
 *
 * @author jimmy
 * @date 13/07/2024 11:11
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysUserManager {

    private final SysFileService fileService;
    private final SysUserService userService;
    private final SysUserDrugService userDrugService;
    private final WmDaDrugEntDrugtableService drgEntDrugtableService;
    private final SysUserCertificateService userCertificateService;


    /**
     * 创建用户证书
     *
     * @param resource 证书信息
     * @param user     操作人
     * @return 创建结果
     */
    public Boolean createUserCertificate(SysUserDTO.OnCertificateCreate resource, HnqzUser user) {


        return userCertificateService.createUserCertificate(resource, user);
    }

    /**
     * 获取用户ID
     *
     * @param userId 用户ID
     * @return 用户证书信息
     */
    public List<Map<String, Object>> listUserCertificates(Integer userId) {
        List<SysUserCertificate> certificates = userCertificateService.list(Wrappers.<SysUserCertificate>lambdaQuery().eq(SysUserCertificate::getUserId, userId));


        if (CollUtil.isEmpty(certificates)) {
            return Collections.emptyList();
        }

        Set<Long> fileIds = certificates.stream().mapToLong(SysUserCertificate::getFileId).boxed().collect(Collectors.toSet());

        Map<Long, SysFile> idToFileMap = fileService.listByIds(fileIds).stream().collect(Collectors.toMap(SysFile::getId, Function.identity()));


        return certificates.stream().map(cert -> {
            Map<String, Object> map = new HashMap<>(6);
            map.put("userId", cert.getUserId());
            Long fileId = cert.getFileId();
            map.put("fileId", fileId);
            map.put("file", idToFileMap.get(fileId));
            map.put("type", cert.getType());
            map.put("createTime", cert.getCreateTime());
            map.put("createBy", cert.getCreateBy());
            return map;
        }).collect(Collectors.toList());

    }

    /**
     * 备案管理员绑定备案信息和产品
     *
     * @param resource 绑定信息
     * @param user     操纵人
     * @return 绑定结果
     */
    public Boolean bindProdToUserCert(SysUserDTO.OnProdBind resource, HnqzUser user) {

        if (CollUtil.isNotEmpty(resource.getProductIds())) {
            String products = resource.getProductIds().stream().map(Objects::toString).collect(Collectors.joining(","));
            userCertificateService.update(Wrappers.<SysUserCertificate>lambdaUpdate()
                    .set(SysUserCertificate::getProducts, products)
                    .eq(SysUserCertificate::getCertificateId, resource.getCertId()));
        }

        // 启用业务员
        userService.update(Wrappers.<SysUser>lambdaUpdate()
                .set(SysUser::getLockFlag, CommonConstants.STATUS_NORMAL)
                .eq(SysUser::getUserId, resource.getUserId())
                .eq(SysUser::getDelFlag, CommonConstants.STATUS_NORMAL)
                .eq(SysUser::getLockFlag, CommonConstants.STATUS_LOCK));


        return Boolean.TRUE;
    }

    /**
     * 获取用户备案详情
     *
     * @param userId 用户ID
     * @return 备案信息
     */
    public List<Map<String, Object>> listUserCerts(Integer userId) {

        List<SysUserCertificate> certificates = userCertificateService.list(Wrappers.<SysUserCertificate>lambdaQuery().eq(SysUserCertificate::getUserId, userId));

        if (CollUtil.isEmpty(certificates)) {
            return Collections.emptyList();
        }

        // 查询药品
        Set<String> prodIds = certificates.stream().filter(cert -> CharSequenceUtil.isNotBlank(cert.getProducts())).flatMap(cert -> CharSequenceUtil.split(cert.getProducts(), StrPool.COMMA).stream()).collect(Collectors.toSet());
        Map<String, WmDaDrugEntDrugtable> collect;
        if (CollUtil.isNotEmpty(prodIds)) {
            collect = drgEntDrugtableService.listByIds(prodIds).stream().collect(Collectors.toMap(WmDaDrugEntDrugtable::getId, Function.identity()));
        } else {
            collect = Collections.emptyMap();
        }

        // 查询图片
        Set<Long> fileIds = certificates.stream().mapToLong(SysUserCertificate::getFileId).boxed().collect(Collectors.toSet());
        Map<Long, SysFile> fileIdToFileMap = fileService.listByIds(fileIds).stream().collect(Collectors.toMap(SysFile::getId, Function.identity()));


        return certificates.stream().map(cert -> {
            Map<String, Object> map = BeanUtil.copyProperties(cert, Map.class, "products", "fileId");

            // 产品
            String products = cert.getProducts();
            List<Map<String, Object>> prods;
            if (CharSequenceUtil.isNotBlank(products)) {
                prods = CharSequenceUtil.split(products, StrPool.COMMA).stream().map(prodId -> {
                    WmDaDrugEntDrugtable wmDaDrugEntDrugtable = collect.get(prodId);
                    Map<String, Object> m = new HashMap<>(3);
                    m.put("productId", wmDaDrugEntDrugtable.getId());
                    m.put("productCommonName", wmDaDrugEntDrugtable.getDrugnameTy());
                    m.put("productnName", wmDaDrugEntDrugtable.getDrugname());
                    return m;
                }).collect(Collectors.toList());
            } else {
                prods = Collections.emptyList();
            }
            map.put("products", prods);

            // 图片
            SysFile f = fileIdToFileMap.get(cert.getFileId());
            map.put("file", f);

            return map;
        }).collect(Collectors.toList());
    }


    @Transactional(rollbackFor = Exception.class)
    public Boolean bindDrug(SysUserDTO.OnProdBind resource, HnqzUser user) {


        // 删除老数据
        userDrugService.remove(Wrappers.<SysUserDrug>lambdaQuery().eq(SysUserDrug::getUserId, resource.getUserId()));

        List<Integer> productIds = resource.getProductIds();

        LocalDateTime now = LocalDateTime.now();
        String username = user.getUsername();

        if (CollUtil.isNotEmpty(productIds)) {
            List<SysUserDrug> collect = productIds.stream().map(prod -> {
                SysUserDrug rel = new SysUserDrug();
                rel.setUserId(resource.getUserId());
                rel.setDrugId(prod);
                rel.setCreateBy(username);
                rel.setCreateTime(now);
                return rel;

            }).collect(Collectors.toList());


            userDrugService.saveBatch(collect);

        }

        return Boolean.TRUE;
    }

    public List<String> listBondProds(Integer userId) {

        return userDrugService.list(Wrappers.<SysUserDrug>lambdaQuery().eq(SysUserDrug::getUserId, userId))
                .stream().map(ud -> ud.getDrugId().toString()).distinct().sorted().collect(Collectors.toList());
    }
}
