package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.entity.SysEnterpriseArea;
import com.qunzhixinxi.hnqz.admin.mapper.SysEnterpriseAreaMapper;
import com.qunzhixinxi.hnqz.admin.service.SysEnterpriseAreaService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 企业区域关系表
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/13 13:44
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysEnterpriseAreaServiceImpl extends ServiceImpl<SysEnterpriseAreaMapper, SysEnterpriseArea> implements SysEnterpriseAreaService {

    /**
     * 保存企业区域关系
     *
     * @param deptId    部门ID
     * @param areaCodes 区域ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveRelation(Long deptId, Collection<Long> areaCodes) {

        if (CollUtil.isEmpty(areaCodes)) {
            log.info("所属区域缺失");
            return;
        }
        LocalDateTime now = LocalDateTime.now();
        List<SysEnterpriseArea> collect = areaCodes.stream().distinct().map(code -> {
            SysEnterpriseArea enterpriseArea = new SysEnterpriseArea();
            enterpriseArea.setEntId(deptId);
            enterpriseArea.setAreaId(code);
            enterpriseArea.setCreateTime(now);
            return enterpriseArea;
        }).collect(Collectors.toList());

        this.saveBatch(collect);

    }

    /**
     * 更新企业区域关系
     *
     * @param deptId    企业ID
     * @param areaCodes 区域ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateRelation(Long deptId, Collection<Long> areaCodes) {
        // 删除旧的
        this.remove(Wrappers.<SysEnterpriseArea>lambdaQuery().eq(SysEnterpriseArea::getEntId, deptId));
        // 插入信息
        LocalDateTime now = LocalDateTime.now();
        List<SysEnterpriseArea> collect = areaCodes.stream().map(code -> {
            SysEnterpriseArea enterpriseArea = new SysEnterpriseArea();
            enterpriseArea.setEntId(deptId);
            enterpriseArea.setAreaId(code);
            enterpriseArea.setCreateTime(now);
            return enterpriseArea;

        }).collect(Collectors.toList());

        this.saveBatch(collect);
    }

    /**
     * 获取企业的区域
     *
     * @param deptId 企业ID
     * @return 区域列表
     */
    @Override
    public List<Long> listEntAreas(Collection<Long> deptId) {
        List<Long> collect = deptId.stream().sorted().collect(Collectors.toList());
        return this.list(Wrappers.<SysEnterpriseArea>lambdaQuery().in(SysEnterpriseArea::getEntId, collect)).stream().map(SysEnterpriseArea::getAreaId).collect(Collectors.toList());
    }


    /**
     * 查询企业的所属区域
     *
     * @param deptId 部门ID
     * @return 映射关系
     */
    @Override
    public Map<Long, List<Long>> listEntAreas1(Collection<Long> deptId) {
        List<Long> collect = deptId.stream().sorted().collect(Collectors.toList());
        return this.list(Wrappers.<SysEnterpriseArea>lambdaQuery().in(SysEnterpriseArea::getEntId, collect)).stream().collect(Collectors.groupingBy(SysEnterpriseArea::getEntId, Collectors.mapping(SysEnterpriseArea::getAreaId, Collectors.toList())));
    }

    /**
     * 删除关系
     *
     * @param deptId 部门ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delRelation(Long deptId) {

        this.remove(Wrappers.<SysEnterpriseArea>lambdaQuery().eq(SysEnterpriseArea::getEntId, deptId));

    }

    @Override
    public List<Long> listAreaEnts(List<Long> areas) {
        List<Long> collect = areas.stream().distinct().sorted().collect(Collectors.toList());
        return this.list(Wrappers.<SysEnterpriseArea>lambdaQuery().in(SysEnterpriseArea::getAreaId, collect)).stream().map(SysEnterpriseArea::getEntId).distinct().sorted().collect(Collectors.toList());
    }
}




